from cqupt import __version__, __description__
from cqupt.log import Log
from cqupt.auth import Auth
from cqupt.urls import URL_LOGIN
from cqupt.request import Request
from cqupt.crawlers import Crawler
from argparse import ArgumentParser, SUPPRESS
from requests import Session, ConnectionError
from sys import argv


class Piper:
    def __init__(self):
        Auth.init()
        self.session: Session = Session()
        self.user: dict = Auth.load_user()
        self.cookie: dict = Auth.read_cookie()

    def authorize(self):
        if not self.user:
            self.user = Auth.add_user()

        if self.cookie is None:
            try:
                self.session.get(URL_LOGIN)
            except ConnectionError:
                Log.fatal('网络走丢啦~')
            
            self.cookie = self.session.cookies.get_dict()
            Auth.save_cookie(self.cookie)
            Request.handle_login(Request.login(self.user, self.cookie))
        else:
            self.session.cookies.update(self.cookie)

    def handle_args(self, args):
        if args.drop:
            self.drop()
        if args.clear:
            self.clear()
        if args.whoami:
            self.show_config()
        if args.auth:
            Auth.update_user('确定要更改绑定吗?')

    def clear(self):
        Auth.clear_cookie()
        Log.fatal('已清除缓存')

    def drop(self):
        Auth.drop_config()
        Log.fatal('已删除配置')

    def show_config(self):
        config: dict = Auth.load_config()
        if config.get('user'):
            print('学号:', config.get('user').get('userid'))
            print('密码:', config.get('user').get('password'))
        exit(0)


def construct_args():
    parser = ArgumentParser(prog='cqupt', description=__description__, )
    parser.add_argument('-v', '--version', action='version', version=__version__)
    
    group_internal = parser.add_argument_group('内置功能')
    group_internal.add_argument('--auth', action='store_true', help='绑定学号')
    group_internal.add_argument('--drop', action='store_true', help='删除配置')
    group_internal.add_argument('--clear', action='store_true', help='清除缓存')
    group_internal.add_argument('--whoami', action='store_true', help='显示配置')

    group_crawler = parser.add_argument_group('从教务在线获取信息')
    group_crawler.add_argument('--fee', metavar='学年', type=int, const=-1, nargs='?', help='获取学年学费')
    group_crawler.add_argument('--gpa', metavar='学年', type=int, const=-1, nargs='?', help='获取学年绩点')
    group_crawler.add_argument('--credit', metavar='学年', type=int, const=-1, nargs='?', help='获取学年学分')
    group_crawler.add_argument('--photo', metavar='学号', type=str, const='self', nargs='?', help='获取学生照片')
    group_crawler.add_argument('--task', action='store_true', help='获取考试安排')
    group_crawler.set_defaults(handle=Crawler().handle)

    return parser.parse_args()


def cli():
    if len(argv) <= 1:
        exit(0)

    piper = Piper()

    args = construct_args()
    # print(args)

    piper.handle_args(args)
   
    piper.authorize()

    if hasattr(args, 'handle'):
        args.handle(piper, args)
    else:
        print('help')
    


from cqupt.urls import URL_FINANCE
from cqupt.log import loading
from prettytable import PrettyTable
from bs4 import BeautifulSoup


class Fee:
    fee: dict = {}
    length: int = 0

    @classmethod
    @loading('正在获取学费信息')
    def crawl(cls, request):
        soup = BeautifulSoup(request.get(URL_FINANCE).text, 'html.parser')
        for tr in soup.find('table', {'class', 'pTable'}).findAll('tr')[1:]:
            tds = tr.findAll('td')
            school_year = tds[0].text.split('-')[0]
            cls.fee[school_year] = list()
            for td in tds:
                cls.fee[school_year].append(td.text)

    @classmethod
    def handle(cls, request, arg):
        cls.crawl(request)
        table = PrettyTable(['学年', '应缴', '已缴', '未缴'])

        if arg == -1:
            for _, v in cls.fee.items():
                table.add_row(v)
                cls.length += 1
        else:
            if str(arg) in cls.fee:
                table.add_row(cls.fee.get(str(arg)))
                cls.length += 1
        
        if cls.length == 0:
            print('\n无查询结果!')
        else:
            print()
            print(table)
            print('具体明细，请查询财务处集中收费平台')



from cqupt import PIPER_HOME
from cqupt.log import Log
from cqupt.fs import b64_fwrite, b64_fread
from time import time
from getpass import getpass
from os import mkdir, path
from shutil import rmtree

PATH_PIPER_COOKIE = f'{PIPER_HOME}/cookie'
PATH_PIPER_USER = f'{PIPER_HOME}/user'

MAX_COOKIE_LIVE = 60 * 15  # 15mins

class Auth:
    @classmethod
    def init(cls):
        if not path.isdir(PIPER_HOME):
            mkdir(PIPER_HOME)
        if not path.isfile(PATH_PIPER_COOKIE):
            b64_fwrite(PATH_PIPER_COOKIE, None)
        if not path.isfile(PATH_PIPER_USER):
            b64_fwrite(PATH_PIPER_USER, None)

    @classmethod
    def drop_config(cls):
        rmtree(PIPER_HOME)

    @classmethod
    def clear_cookie(cls):
        cls.save_cookie(None)

    @classmethod
    def load_config(cls) -> dict:
        return {
            'user': cls.load_user(),
            'cookie': cls.read_cookie()
        }

    @classmethod
    def read_cookie(cls) -> dict or None:
        cookie: dict = eval(b64_fread(PATH_PIPER_COOKIE))
        # The math here can't be wrong!
        if cookie and int(time()) - cookie.get('ts') < MAX_COOKIE_LIVE:
            return cookie.get('cookie')
        return None

    @classmethod
    def save_cookie(cls, cookie: dict):
        b64_fwrite(PATH_PIPER_COOKIE, {
            'cookie': cookie,
            'ts': int(time())
        })

    @classmethod
    def load_user(cls) -> dict or None:
        user: dict = eval(b64_fread(PATH_PIPER_USER))
        return user

    @classmethod
    def save_user(cls, user: dict):
        b64_fwrite(PATH_PIPER_USER, user)

    @classmethod
    def add_user(cls):
        print('为确保本人操作, 第一次登录需输入学号密码')
        return cls.enter_user()

    @classmethod
    def update_user(cls, message: str = '要更改绑定吗?'):
        if input(f'{message} [y/N]: ') != 'N':
            cls.enter_user()
            Log.fatal('更改成功')
        else:
            Log.fatal('运行命令 cqupt --auth 可以更改密码哦~')

    @classmethod
    def enter_user(cls):
        user: dict = {}
        user['userid'] = input('学号: ')
        user['password'] = getpass(prompt='密码: ')
        cls.save_user(user)
        return user



URL_LOGIN='http://jwzx.cqu.pt/login.php'
URL_CAPTCHA='http://jwzx.cqu.pt/createValidationCode.php'
URL_CHECK_LOGIN='http://jwzx.cqu.pt/checkLogin.php'

# 各类数据在 HTML 的标签id: # 原始成绩总表: id='cjAllTab-cjzb'
# 排名和绩点: id='cjAllTab-zypm'
# 各类统考成绩: id='ui-id-5'
URL_GPA='http://jwzx.cqu.pt/student/chengjiPm.php'

# 各类数据在 HTML 的标签id: # 学分统计表: id='AxfTjTable'
URL_CREDIT='http://jwzx.cqu.pt/student/xkxfTj.php'

# 各类数据在 HTML 的标签id: # 学费: id='pTable'
URL_FINANCE='http://jwzx.cqu.pt/student/jfInfo.php'

# 考试安排 #id='stuKsTab-ks'
URL_TASKS='http://jwzx.cqu.pt/student/ksap.php'

# 学生照片 
URL_STUDENT_PHOTO='http://jwzx.cqu.pt/showstupic.php?xh='


from cqupt.urls import URL_TASKS
from cqupt.log import loading
from prettytable import PrettyTable
from bs4 import BeautifulSoup


class Task:
    task: list = []

    @classmethod
    @loading('正在获取考试安排')
    def crawl(cls, request):
        soup = BeautifulSoup(request.get(URL_TASKS).text, 'html.parser')
        for tr in soup.find('tbody').findAll('tr'):
            tds = tr.findAll('td')[5:]
            cls.task.append([
                tds[0].text,  # 课程名称
                tds[4].text,  # 教室号
                tds[5].text,  # 座位号
                tds[1].text,  # 周次
                tds[2].text,  # 星期
                tds[3].text,  # 具体时间
                tds[6].text   # 资格
            ])

    @classmethod
    def handle(cls, request, arg):
        cls.crawl(request)
        row = None
        table = PrettyTable(['课程名称', '教室', '座位', '周次', '星期', '具体时间', '资格'])
        if cls.task:
            for row in cls.task:
                table.add_row(row)
        if cls.task:
            print('\n共', len(cls.task), '项考试')
            print(table)
            print('多喝热水, 及时做好复习准备哦~')
        else:
            print('\n无查询结果!')


from cqupt.urls import URL_STUDENT_PHOTO
from cqupt.auth import Auth
from cqupt.log import loading
from cqupt import PIPER_HOME
from threading import Thread
from PIL import Image
from os import remove


white_list: dict = {
    '2017213056': 'https://raw.githubusercontent.com/Mivinci/cqupt-piper/master/assert/th.jpeg'
}


class Photo:

    @classmethod
    def handle(cls, request, arg):
        user: dict = Auth.load_user()
        stuid: str = user.get('userid') if arg == 'self' else arg
        path: str = f'{PIPER_HOME}/photo_{stuid}.jpg'

        with open(path, 'wb') as f:
            f.write(cls.get_photo(request, stuid))
        
        Thread(target=cls.show_photo, args=(path,)).start()

    @classmethod
    def show_photo(cls, path: str):
        photo = Image.open(path)
        photo.show()
        remove(path)

    @classmethod
    @loading('正在获取照片')
    def get_photo(cls, request, stuid: str):
        url: str = ''
        if stuid in white_list:
            url = white_list.get(stuid)
        else:
            url= f'{URL_STUDENT_PHOTO}{stuid}'
        return request.get(url).content

        
        
        


from cqupt.urls import URL_GPA
from cqupt.log import loading
from prettytable import PrettyTable
from bs4 import BeautifulSoup


class GPA:
    gpa: dict = {}
    length: int = 0

    @classmethod
    @loading('正在获取绩点排名信息')
    def crawl(cls, request):
        soup = BeautifulSoup(request.get(URL_GPA).text, 'html.parser')
        for tr in soup.find('div', {'id': 'cjAllTab-zypm'}).findAll('tr')[1::3]:
            tds: list = tr.findAll('td')
            year = tds[0].text
            cls.gpa[year] = [tds[0].text, tds[2].text, tds[3].text]

    @classmethod
    def handle(cls, request, arg):
        cls.crawl(request)
        table = PrettyTable(['学年', '绩点', '排名'])

        if arg == -1:
            for _, v in cls.gpa.items():
                table.add_row(v)
                cls.length += 1
        else:
            if len(str(arg)) == 4:
                for year in [f'{arg}1', f'{arg}2']:
                    if year in cls.gpa:
                        table.add_row(cls.gpa.get(year))
                        cls.length += 1
        if cls.length == 0:
            print('\n无查询结果!')
        else:
            print()
            print(table)


import base64
from os import remove


def b64enc(s: any) -> str:
    if not isinstance(s, str):
        s = str(s)
    return base64.b64encode(str.encode(s)).decode()


def b64dec(s: str) -> str:
    return base64.b64decode(s).decode()


def b64_fwrite(filename: str, data: any) -> int:
    try:
        with open(filename, 'w') as f:
            return f.write(b64enc(data))
    except FileExistsError:
        raise


def b64_fread(filename: str) -> str:
    try:
        with open(filename, 'r') as f:
            return b64dec(f.read())
    except FileExistsError:
        remove(filename)
        raise


from pathlib import Path


__name__ = 'CQUPT Piper'
__generation__ = '0.0.12'
__version__ = f'v{__generation__}-light'
__author__ = 'XJJ'
__description__ = f'{__name__} {__version__} @{__author__}'

PIPER_HOME = f'{Path.home()}/.piper'



from cqupt.urls import URL_CREDIT
from cqupt.log import loading
from prettytable import PrettyTable
from bs4 import BeautifulSoup


class Credit:
    credit: dict = {}
    length: int = 0

    @classmethod
    @loading('正在获取学分信息')
    def crawl(cls, request):
        soup = BeautifulSoup(request.get(URL_CREDIT).text, 'html.parser')
        for tr in soup.find('table', {'id': 'AxfTjTable'}).findAll('tr')[1:]:
            tds: list = tr.findAll('td')
            year = tds[1].text
            cls.credit[year] = []
            for td in tds[1:5]:
                cls.credit[year].append(td.text)

    @classmethod
    def handle(cls, request, arg):
        cls.crawl(request)
        table = PrettyTable(['学年', '学分', '二专业学分', '统计时间'])

        if arg == -1:
            for _, v in cls.credit.items():
                table.add_row(v)
                cls.length += 1
        else:
            if len(str(arg)) == 4:
                for year in [f'{arg}1', f'{arg}2']:
                    if year in cls.credit:
                        table.add_row(cls.credit.get(year))
                        cls.length += 1
            else:
                if str(arg) in cls.credit:
                    table.add_row(cls.credit.get(str(arg)))
                    cls.length += 1
        
        if cls.length == 0:
            print('\n无查询结果!')
        else:
            print()
            print(table)


        


from os import system
from sys import stdout, platform
from time import sleep
from threading import Thread


class Log:
    prefix = '\ncqupt:'

    @classmethod
    def error(cls, message: str):
        print(cls.prefix, message)

    @classmethod
    def fatal(cls, message: str = '', status: int = 0):
        cls.error(message)
        exit(status)


class Loading:
    frames = ['[=   ]', '[ =  ]', '[  = ]', '[   =]']
    curr_i = -1
    done = False

    @classmethod
    def start(cls, msg: str):
        while not cls.done:
            if cls.curr_i < 3:
                cls.curr_i += 1
            else:
                cls.curr_i = 0
            if platform == 'win32':
                print(cls.frames[cls.curr_i], msg, end='\r')
            else:
                print(cls.frames[cls.curr_i], msg)
                stdout.write("\033[F")
            sleep(0.2)


def loading(message: str):
    def loading_wrraper(target):
        def target_wrapper(*args, **kwargs):
            Thread(target=Loading.start, args=(message,)).start()
            resp = target(*args, **kwargs)
            Loading.done = True
            stdout.flush()
            return resp
        return target_wrapper
    return loading_wrraper




from requests import post, ConnectionError
from cqupt.log import Log, loading, Loading
from cqupt.auth import Auth


# URL_LOGIN_API = 'http://127.0.0.1:5000/login'
URL_LOGIN_API = 'http://47.100.139.100:5000/login'


class Request:

    @staticmethod
    @loading('登录过期, 正在重新请求登录')
    def login(user: dict, cookie: dict) -> dict:
        user.update(cookie)
        try:
            return eval(post(URL_LOGIN_API, data=user).text)
        except ConnectionError:
            Loading.done = True
            Auth.clear_cookie()
            Log.fatal('服务器走丢啦~')

    @staticmethod
    def handle_login(resp: dict):
        if resp.get('code') == 401:
            print('\n' + resp.get('msg'))
            Auth.update_user()
            Auth.clear_cookie()
        elif resp.get('code') == 200:
            print('\n\033[92m登录成功\033[0m')
        else:
            Auth.clear_cookie()
            Log.fatal(resp.get('msg'))



from setuptools import setup, find_packages
from cqupt import __generation__

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='cqupt',
    version=__generation__,
    description='CQUPT Piper is a command line tool to get info from jwzx.cqupt.edu.cn',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Mivinci XJJ',
    url='https://github.com/mivinci/cqupt-piper',
    packages=find_packages(),
    install_requires=[
        'PrettyTable',
        'Pillow',
        'requests',
        'bs4'
    ],
    python_requires='>=3.5',
    entry_points={
        'console_scripts': [
            'cqupt = cqupt.piper:cli'
        ]
    }
)



from cqupt.crawlers.fee import Fee
from cqupt.crawlers.credit import Credit
from cqupt.crawlers.gpa import GPA
from cqupt.crawlers.task import Task
from cqupt.crawlers.photo import Photo


class Crawler:
    def __init__(self): pass

    def handle(self, piper, argument):
        if argument.credit:
            Credit.handle(piper.session, argument.credit)
        if argument.gpa:
            GPA.handle(piper.session, argument.gpa)
        if argument.task:
            Task.handle(piper.session, argument.task)
        if argument.fee:
            Fee.handle(piper.session, argument.fee)
        if argument.photo:
            Photo.handle(piper.session, argument.photo)